package in_memory

import (
	"downloader/internal/core/entities"
	"downloader/internal/core/ports/download_request"
	"errors"
	"go.uber.org/zap"
)

type inMemoryRepository struct {
	collection map[string]*entities.Download
	logger     *zap.SugaredLogger
}

func NewInMemoryRepository(logger *zap.SugaredLogger) download_request.Repository {
	return &inMemoryRepository{collection: make(map[string]*entities.Download), logger: logger}
}

func (i *inMemoryRepository) Create(download *entities.Download) (*entities.Download, error) {
	logger := i.logger.With("downloadId", download.ID)

	if doc := i.collection[download.ID]; doc != nil {
		logger.Warn("create: download already exists")
		return nil, errors.New("download request already exists")
	}

	i.collection[download.ID] = download
	logger.Info("added download to database")

	return download, nil
}

func (i *inMemoryRepository) Update(download *entities.Download) error {
	logger := i.logger.With("downloadId", download.ID)

	if doc := i.collection[download.ID]; doc == nil {
		logger.Warn("update: download doesnt exist")
		return errors.New("download request doesn't exist exists")
	}

	i.collection[download.ID] = download
	logger.Info("updated download to database")

	return nil
}

func (i *inMemoryRepository) GetById(id string) (*entities.Download, error) {
	logger := i.logger.With("downloadId", id)

	if download := i.collection[id]; download != nil {
		return download, nil
	} else {
		logger.Warn("download was not found in the database")
		return nil, errors.New("download was not found in the database")
	}
}
